#    Police Operation Garden Exhibition
#    Copyright (C) 2016  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pygame

from code.classes.HairCross import HairCross
from code.classes.Visitor import Visitor
from code.classes.Area import Area
from code.classes.Player import Player

pygame.init()


class Game:
	"""This class represents the game and the main game routine.

	It is instantinated inside the main() function.

	"""
	def __init__( self, _game_window_settings,
					 	_game_clock_settings,
					 	_game_cursor_settings,
					 	_hair_cross_settings,
					 	_visitor_settings,
					 	_area_settings,
					 	_player_settings ):
		"""Initialises the class.

		"""
		# These are the objects / variables that will be used in this class
		# and are accessible from outside.
		# Some of them will be initialised by the "init_*" functions.
		self.window = None
		self.clock = None
		self.running = None
		self.events = None

		self.hair_cross = None
		self.area = None

		self.visitors = pygame.sprite.Group()

		# We are going to save some settings as we are going to need some of
		# the settings later on during the game, such the as frame rate of
		# the game clock.
		self.game_window_settings = _game_window_settings
		self.game_clock_settings = _game_clock_settings
		self.game_cursor_settings = _game_cursor_settings

		self.hair_cross_settings = _hair_cross_settings
		self.visitor_settings = _visitor_settings
		self.area_settings = _area_settings
		self.player_settings = _player_settings

		# Now we initialise some things so we can get started with the game.
		self.init_window()
		self.init_clock()
		self.init_cursor()
		self.init_area_class()
		self.init_area()
		self.init_visitor_class()
		self.init_player()
		
		self.greything = pygame.sprite.Sprite()
		self.greything.image = pygame.Surface( ( 1016, 768 ), pygame.SRCALPHA )
		self.greything.image.set_alpha( 100 )
		self.greything.image.fill( ( 150, 150, 150, 200 ))
		self.greything.rect = self.greything.image.get_rect()
		self.greything.rect.topleft = ( 350, 0 )


	def init_window( self ):
		"""Initialises the game window where all the game action is happening.

		"""
		self.window = pygame.display.set_mode( self.game_window_settings["resolution"],
											   *self.game_window_settings["flags"] )


	def init_clock( self ):
		"""Initialises the game clock which controls the update interval of the
		main game loop.

		"""
		self.clock = pygame.time.Clock()


	def init_cursor( self ):
		"""Sets the cursor to being either visible or hidden.

		"""
		pygame.mouse.set_visible( self.game_cursor_settings["visible"] )


	def init_visitor_class( self ):
		"""This function initialises visitor one.

		"""
		Visitor.init_class_variables( self.visitor_settings, Area.collision_rectangles )


	def init_area_class( self ):
		"""This function initialises the area class.

		"""
		Area.init_class_variables( self.area_settings )


	def init_area( self ):
		"""This function initialises the area.

		"""
		self.area = Area( self.area_settings )


	def init_player( self ):
		"""Initialises the player.
		
		"""
		self.player = Player( self.player_settings )


	def check_quit_gameloop( self ):
		"""This function checks whether the user has pressed the "ESC" button.
		If so then self.running is set to False.

		"""
		for e in self.events:
			if e.type == pygame.KEYDOWN and e.key == pygame.K_ESCAPE:
				self.running = False


	def run_menu( self ):
		"""

		"""
		self.running = True

		self.window.fill( self.game_window_settings["background_color"] )
		self.window.blit( self.area.image,
						  self.area.rect )
		self.window.blit( self.area.surface_left.image,
						  self.area.surface_left.rect )
		Visitor.visitors.draw( self.window )
		self.window.blit( self.player.image,
						  self.player.rect )
		self.window.blit( self.player.infopanel.image,
						  self.player.infopanel.rect )
		self.window.blit( self.greything.image, self.greything.rect )		
		
		
		while self.running:
			self.events = pygame.event.get()

			for e in self.events:
				if e.type == pygame.KEYDOWN:
					if e.key == pygame.K_RETURN:
						# Now we initialise some things so we can get started with the game.
						self.run()
						self.running = True
						self.reinit()
					elif e.key == pygame.K_ESCAPE:
						self.running = False
				
			self.window.blit( self.greything.image, self.greything.rect )
			pygame.display.update()	
			self.clock.tick( self.game_clock_settings["fps"] )	

	def reinit( self ):	
		"""

		"""
		self.init_cursor()
		self.init_area_class()
		self.init_area()
		self.init_visitor_class()
		self.init_player()


	def run( self ):
		"""This function runs the game (loop).

		"""
		self.running = True
		
		
		while self.running:

			if self.player.acci_hit > 15 or self.player.faulty_ones > 15:
				self.running = False
		
			self.events = pygame.event.get()
			
			self.check_quit_gameloop()

			# Update section
			self.area.update()
			
			Visitor.update_creation_loop() # Maybe create new visitors.
			Visitor.visitors.update( self.player ) # update all visitors.

			self.player.update( self.events, Visitor )			
				
			
			# Draw section
			self.window.fill( self.game_window_settings["background_color"] )
			self.window.blit( self.area.image,
							  self.area.rect )
			self.window.blit( self.area.surface_left.image,
							  self.area.surface_left.rect )
			Visitor.visitors.draw( self.window )
			self.window.blit( self.player.image,
							  self.player.rect )
			self.window.blit( self.player.infopanel.image,
							  self.player.infopanel.rect )
			
			pygame.display.update()
			
			self.clock.tick( self.game_clock_settings["fps"] )
		
