#    Police Operation Garden Exhibition
#    Copyright (C) 2016  Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import copy
import os.path
import random
import math
import pygame
from code.classes.Animation import Animation
from code.modules import loading

pygame.init()


class Visitor( pygame.sprite.Sprite ):
	"""This class represents a visitor who wants to visit the garden exhibition.

	"""
	# Class variables.
	visitor_settings = None
	visitor_creation_settings = None
	visitors_settings = None
	
	interval_range = None
	creation_counter = None
	creation_interval = None

	visitors = pygame.sprite.Group()

	area_collision_rectangles = None

	
	@staticmethod
	def init_class_variables( _visitor_settings, _area_collision_rectangles ):
		"""Initialises class variables.

		"""
		Visitor.visitor_settings = _visitor_settings
		Visitor.visitors.empty()
		
		Visitor.visitor_creation_settings = _visitor_settings["visitor_creation_settings"]
		Visitor.interval_range = Visitor.visitor_creation_settings["interval_range"]
		Visitor.max_interval_range = Visitor.visitor_creation_settings["max_interval_range"]
		Visitor.min_interval_range = Visitor.visitor_creation_settings["min_interval_range"]
		Visitor.set_new_creation_interval()

		Visitor.visitors_settings = _visitor_settings["visitors"]

		Visitor.area_collision_rectangles = _area_collision_rectangles


	@staticmethod
	def set_new_creation_interval():
		"""Sets a new creation interval and resets the creation counter.

		"""
		Visitor.creation_counter = 0

		if Visitor.max_interval_range >= ( Visitor.min_interval_range + 5 ):
			Visitor.max_interval_range -= 5
			Visitor.creation_interval = Visitor.max_interval_range


	@staticmethod
	def update_creation_loop():
		"""Updates the creation loop.

		"""
		Visitor.creation_counter += 1
		if Visitor.creation_counter >= Visitor.creation_interval:
			if len( Visitor.visitors ) < 10:
				Visitor.create_new()
				Visitor.set_new_creation_interval()

		
	@staticmethod
	def create_new():
		"""This method creates and returns a new instance of this class, and
		also sets a new creation interval to create more instance later on.

		"""
		Visitor.visitors.add(
			Visitor(
				random.choice( Visitor.visitors_settings )
			)
		)


	@staticmethod
	def check_routine_hit( _player ):
		"""Checks if any robots have been hit and performs appropriate action.

		"""
		for v in Visitor.visitors:
			if _player.rect.colliderect( v.rect ):
				if _player.rect.contains( v.rect ):
					if v.state == "robot_python_fix" or v.state == "robot_php_fix" or v.state == "robot_javascript_fix":
						if _player.player_hit_setting == v.state:
							v.fix_robot()
							_player.score += 20
							_player.fixed += 1
							choice = random.randint( 1, 2 )
							if choice == 1:
								_player.sound_and_another_one_fixed.play()
							if choice == 2:
								_player.sound_good_job.play()
						else:
							_player.sound_wrong_source_code.play()
								
					elif _player.player_hit_setting == "kill":
						if v.state == "robot_ok":
							_player.score -= 20
							_player.acci_hit += 1	
							choice = random.randint( 1, 2 )
							if choice == 1:
								_player.sound_sorry_mate.play()		
							if choice == 2:
								_player.sound_what_am_i_doing.play()
						else:
							_player.score += 10
							choice = random.randint( 1, 2 )
							if choice == 1:
								_player.sound_well_done.play()
							elif choice == 2:
								_player.sound_alright.play()
						
						v.kill() # Killing it nontheless.
				else:
					choice = random.randint( 1, 2 )
					if choice == 1:
						_player.sound_ah_almost.play()
					elif choice == 2:
						_player.sound_hmm_not_quite.play()

	@staticmethod
	def check_routine_analyse( _rect ):
		"""Checks if any robots have been hit and performs appropriate action.

		"""
		for v in Visitor.visitors:
			if _rect.contains( v.rect ):
				v.showing_state = True
		
		
	def __init__( self, _visitor_settings ):
		"""This is the constructor function and initialises a visitor.

		"""
		super( Visitor, self ).__init__()

		# Variables and objects which will be used throughout the class.
		self.animation = None
		self.speed_x_buffer = 0
		self.speed_y_buffer = 0
		self.speed_x = 0
		self.speed_y = 0
		self.direction = 0
		self.speed = None

		self.state_image = None
		self.state_image_rect = None
		self.state = None
		self.state_timespan = None
		self.state_timespan_counter = None
		self.states = None		
		self.showing_state = None

		self.collision_point_north = pygame.Rect( 0, 0, 20, 20 )

		self.visitor_settings = _visitor_settings


		self.init_speed()
		self.init_animation()
		self.init_sprite()
		self.init_direction()
		self.init_state_settings()
		


	def init_animation( self ):
		"""Initialises the animation object.

		"""
		self.animation = Animation( self.visitor_settings["animation"]["dir_path"],
									self.visitor_settings["animation"]["images"],
									self.visitor_settings["animation"]["step"],
									self.visitor_settings["animation"]["interval"] - float(self.speed))


	def init_sprite( self ):
		"""Sets the initial image and rect so we can draw on the window.

		"""
		self.image = self.animation.image
		self.rect = self.image.get_rect()

		start_pos_choice = self.visitor_settings["start_positions"][1]
		self.rect.center = ( random.randint( *start_pos_choice["start_pos_x_range"] ),
							 random.randint( *start_pos_choice["start_pos_y_range"] ) )


	def init_direction( self ):
		"""Initialises the initial direction (move upwards).

		"""
		self.direction = self.visitor_settings["direction"]


	def init_speed( self ):
		"""Initialises the speed.

		"""
		self.speed = float(random.randint( *self.visitor_settings["speed"] )) / 10


	def init_state_settings( self ):
		"""Initialises the state settings.

		"""
		arr = None

		self.states = self.visitor_settings["states"].keys()
		
		self.state = random.choice( self.states )
		self.state_image = loading.load_image(
			os.path.join(
				self.visitor_settings["dir_path"],
				self.visitor_settings["states"][self.state]
			)
		)

		self.state_image_rect = self.image.get_rect()
		self.state_timespan = self.visitor_settings["show_state_timespan"]

		self.state_timespan_counter = 0
		self.showing_state = False


	def fix_robot( self ):
		"""Fixes a robot and makes it useable again.

		"""
		self.state = "robot_ok"
		self.state_image = loading.load_image(
			os.path.join(
				self.visitor_settings["dir_path"],
				self.visitor_settings["states"]["robot_ok"]
			)
		)

		self.state_image_rect = self.image.get_rect()


	def update( self, _player ):
		"""Updates the visitor.

		"""
		
		if self.rect.top > 768:
			self.kill()
			if self.state != "robot_ok":
				_player.faulty_ones += 1			

		self.orig_pos = self.rect.center
		self.animation.update()

		if self.showing_state and not self.state_timespan_counter > self.state_timespan:
			self.image = self.state_image
			self.state_timespan_counter += 1
		else:
			self.image = self.animation.image
			self.state_timespan_counter = 0
			self.showing_state = False
			
		self.rect = self.image.get_rect()
		self.rect.center = self.orig_pos

		self.speed_y = self.speed

		rect2 = copy.deepcopy( self.rect )
		rect2.centery += 10

		for v in Visitor.visitors:
			if self == v:
				continue
			else:
				if rect2.colliderect( v.rect ):
					return
		
		if self.speed_y_buffer >= 1:
			self.rect.centery += self.speed_y_buffer
			self.speed_y_buffer = 0
		else:
			self.speed_y_buffer += self.speed_y
